@@ -26,8 +26,8 @@ class IdentityKey(models.Model):
     cipher = models.CharField(max_length=88)
     salt = models.CharField(max_length=44)
     active = models.BooleanField(default=False)
-    timestamp = models.CharField(max_length=100)
-    dt_timestamp = models.DateTimeField(auto_now_add=True)
+    # timestamp = models.CharField(max_length=100)
+    # dt_timestamp = models.DateTimeField(auto_now_add=True)
 
 
 class SignedPreKey(models.Model):
@@ -42,7 +42,7 @@ class SignedPreKey(models.Model):
     salt = models.CharField(max_length=44)
     active = models.BooleanField(default=False)
     # timestamp = models.CharField(max_length=100)
-    dt_timestamp = models.DateTimeField(auto_now_add=True)
+    # dt_timestamp = models.DateTimeField(auto_now_add=True)
 
     class Meta:
         constraints = [models.UniqueConstraint(fields=['keyId', 'user'], name='unique_signed_prekey')]
@@ -58,7 +58,7 @@ class PreKey(models.Model):
     used = models.BooleanField(default=False)
     cipher = models.CharField(max_length=88)
     salt = models.CharField(max_length=44)
-    dt_timestamp = models.DateTimeField(auto_now_add=True)
+    # dt_timestamp = models.DateTimeField(auto_now_add=True)
 
     class Meta:
         constraints = [models.UniqueConstraint(fields=['keyId', 'user'], name='unique_prekey')]
